from setuptools import setup

setup(name='eds_ds_tools',
      version='0.7',
      description='tools for data science',
      packages=['eds_ds_tools'],
      author_email='wenleicao@gmail.com',
      zip_safe=False,      
      )
